import logging

from airflow.contrib.hooks.gcp_api_base_hook import GoogleCloudBaseHook


logger = logging.getLogger(__name__)


class GSCredentials(GoogleCloudBaseHook):
    def __init__(self, gcp_conn_id="google_cloud_default", delegate_to=None):
        super(GSCredentials, self).__init__(gcp_conn_id, delegate_to)

    def get_credentials(self):
        try:
            return self._get_credentials()
        except (Exception):
            logger.exception(
                "Failed to load GCP credentials using connection id - "
                + self.gcp_conn_id
            )
            raise



import logging
import os

from dbnd._core.configuration.environ_config import in_quiet_mode
from dbnd._core.errors.friendly_error.versioned_dagbag import (
    failed_to_load_versioned_dagbag_plugin
)
from dbnd._core.utils.object_utils import patch_module_attr
from dbnd_airflow.constants import AIRFLOW_VERSION_2

logger = logging.getLogger(__name__)


# WE SHOULD NOT HAVE ANY dbnd imports here -- circle import:  dbnd_airflow -> airflow -> load .. -> import dbnd_airflow

def _use_databand_airflow_dagbag():
    """
    Overriding Airflow Dagbag, so versioned dags can be used
    :return:
    """
    import airflow
    from airflow import settings
    if AIRFLOW_VERSION_2 or not settings.RBAC:
        from airflow.www import views
    else:
        from airflow.www_rbac import views

    from dbnd_airflow.web.databand_versioned_dagbag import (
        DbndAirflowDagBag,
        DbndDagModel
    )

    if os.environ.get("SKIP_DAGS_PARSING") != "True":
        views.dagbag = DbndAirflowDagBag(settings.DAGS_FOLDER)
    else:
        views.dagbag = DbndAirflowDagBag(os.devnull, include_examples=False)

    # some views takes dag from dag model
    if hasattr(views, "DagModel"):
        patch_module_attr(views, "DagModel", DbndDagModel)

    # dag_details invoke DagBag directly
    patch_module_attr(airflow.models, "DagBag", DbndAirflowDagBag)
    patch_module_attr(airflow.models.dag, "DagBag", DbndAirflowDagBag)
    patch_module_attr(airflow.models.dag, "DagModel", DbndDagModel)

    logging.info("Using DBND DagBag with support for versioned dags and historic dag runs.")


def use_databand_airflow_dagbag():
    try:
        _use_databand_airflow_dagbag()
    except Exception as e:
        raise failed_to_load_versioned_dagbag_plugin(e)


def patch_airflow_create_app():
    if not in_quiet_mode():
        logger.debug("Adding support for versioned DBND DagBag")

    def patch_create_app(create_app_func):
        def patched_create_app(*args, **kwargs):
            res = create_app_func(*args, **kwargs)
            use_databand_airflow_dagbag()
            return res

        return patched_create_app
    if AIRFLOW_VERSION_2:
        from airflow.www import app as airflow_app_rbac
    else:
        from airflow.www_rbac import app as airflow_app_rbac

    airflow_app_rbac.create_app = patch_create_app(airflow_app_rbac.create_app)

    from airflow.www import app as airflow_app_no_rbac
    airflow_app_no_rbac.create_app = patch_create_app(airflow_app_no_rbac.create_app)



# Please do not any airflow imports in this file
# We want to allow importing constants from this file without any side effects

# Do not change this name unless you change the same constant in monitor_as_dag.py in dbnd-airflow-monitor
MONITOR_DAG_NAME = "databand_airflow_monitor"



import flask

from dbnd_airflow.export_plugin.api_functions import (
    get_dag_runs_states_data,
    get_full_dag_runs_for_plugin,
    get_last_seen_values,
    get_meta,
    get_new_dag_runs,
)
from dbnd_airflow.export_plugin.utils import json_response


def convert_url_param_value_to_list(
    param_name, value_type, default_value, separator=","
):
    if param_name not in flask.request.values:
        return default_value

    param_value = flask.request.values.get(param_name, type=str)
    if not param_value:
        return default_value

    return list(map(value_type, param_value.split(separator)))


def process_metadata_request():
    return json_response(get_meta({}).as_dict())


def process_last_seen_values_request():
    return json_response(get_last_seen_values().as_dict())


def process_new_runs_request():
    last_seen_dag_run_id = flask.request.values.get("last_seen_dag_run_id", type=int)
    last_seen_log_id = flask.request.values.get("last_seen_log_id", type=int)
    extra_dag_runs_ids = convert_url_param_value_to_list("extra_dag_runs_ids", int, [])
    dag_ids = convert_url_param_value_to_list("dag_ids", str, None)
    # default to true
    include_subdags = flask.request.values.get("include_subdags", "").lower() != "false"

    return json_response(
        get_new_dag_runs(
            last_seen_dag_run_id,
            last_seen_log_id,
            extra_dag_runs_ids,
            dag_ids,
            include_subdags,
        ).as_dict()
    )


def process_full_runs_request():
    dag_run_ids = convert_url_param_value_to_list("dag_run_ids", int, [])
    include_sources = flask.request.values.get("include_sources", "").lower() == "true"

    return json_response(
        get_full_dag_runs_for_plugin(dag_run_ids, include_sources).as_dict()
    )


def process_dag_run_states_data_request():
    dag_run_ids = convert_url_param_value_to_list("dag_run_ids", int, [])

    return json_response(get_dag_runs_states_data(dag_run_ids).as_dict())



from targets.connections import build_conn_path


def get_conn_path(conn_id):
    from airflow.hooks.base_hook import BaseHook
    from airflow.models import Connection

    connection = BaseHook.get_connection(conn_id)  # type: Connection
    return build_conn_path(
        conn_type=connection.conn_type,
        hostname=connection.host,
        port=connection.port,
        path=connection.schema,
    )
